@@ -5,7 +5,6 @@ use Pcore::Ext;
 use Pcore::Share::WWW;
 use WWW::CDN;
 use Pcore::Util::Data qw[to_json];
-use Pcore::Util::Text qw[mark_raw];
 
 with qw[Pcore::App::Controller];
 
@@ -15,10 +14,12 @@ has ext_default_theme_classic => ( is => 'ro', isa => Str, default => 'triton' )
 has ext_default_theme_modern  => ( is => 'ro', isa => Str, default => 'triton' );
 has ext_default_locale        => ( is => 'ro', isa => Str, default => 'en' );
 
+has cache => ( is => 'ro', isa => ScalarRef, init_arg => undef );
+
 sub BUILD ( $self, $args ) {
     Pcore::Ext->SCAN( $self->{app}, 'classic' );
 
-    die qq[invalid Ext app] if !$Pcore::Ext::CFG->{app}->{ $self->ext_app };
+    die qq[Ext app "$self->{ext_app}" not found] if !$Pcore::Ext::CFG->{app}->{ $self->ext_app };
 
     return;
 }
@@ -37,7 +38,6 @@ sub run ( $self, $req ) {
     return;
 }
 
-# TODO fix Ext overrides version
 around run => sub ( $orig, $self, $req ) {
 
     # if path tail is not empty - fallback to the original method
@@ -45,7 +45,7 @@ around run => sub ( $orig, $self, $req ) {
 
         # .js file request
         if ( $req->{path_tail} && $req->{path_tail} =~ /\A(.+)[.]js\z/sm ) {
-            my $class = $Pcore::Ext::CFG->{class}->{ $Pcore::Ext::NS . $1 };
+            my $class = $Pcore::Ext::CFG->{class}->{"$Pcore::Ext::NS.$1"};
 
             if ( !$class ) {
                 $req->(404)->finish;
@@ -63,6 +63,13 @@ around run => sub ( $orig, $self, $req ) {
         return;
     }
 
+    # return cached content
+    if ( $self->{cache} ) {
+        $req->( 200, [ 'Content-Type' => 'text/html; charset=UTF-8' ], $self->{cache} )->finish;
+
+        return;
+    }
+
     my $resources = [];
 
     # FontAwesome
@@ -75,7 +82,7 @@ around run => sub ( $orig, $self, $req ) {
     my $ext_resources;
 
     # get theme from query
-    if ( $req->{env}->{QUERY_STRING} =~ /\btheme=([a-z-]+)/sm ) {
+    if ( $req->{env}->{QUERY_STRING} =~ /\btheme=([[:lower:]-]+)/sm ) {
         my $theme = $1;
 
         $ext_resources = WWW::CDN->extjs( $ver, $framework, $theme, $self->{app}->{devel} );
@@ -96,13 +103,30 @@ around run => sub ( $orig, $self, $req ) {
     push $resources->@*, WWW::CDN->extjs_locale( $ver, $framework, $locale, $self->{app}->{devel} );
 
     # Ext overrides
-    push $resources->@*, q[<script type="text/javascript" src="/static/ext/pcore/overrides-v6.js"></script>];
+    my $overrides;
+
+    my $class = eval {
+        my $over = $ver =~ s/[.]/_/smgr;
+
+        P->class->load("Pcore::Ext::Override::$over");
+    };
+
+    if ( !$@ ) {
+        $overrides = $class->overrides;
+
+        if ( !$self->{app}->{devel} ) {
+            require JavaScript::Packer;
+
+            my $js_packer = JavaScript::Packer->init;
+
+            $js_packer->minify( \$overrides, { compress => 'obfuscate' } );    # clean
+        }
+    }
 
     my $loader_path = {
-        q[*]     => '.',
-        Pcore    => '.',
-        Ext      => '/static/ext/src/',
-        'Ext.ux' => '/static/ext/ux/',
+        $Pcore::Ext::NS => '.',
+        Ext             => '/static/ext/src/',
+        'Ext.ux'        => '/static/ext/ux/',
     };
 
     $self->{app}->{api}->{map}->extdirect_map(
@@ -118,16 +142,17 @@ around run => sub ( $orig, $self, $req ) {
                 resources => $resources,
                 ext       => {
                     api_map => $api_map ? to_json($api_map)->$* : undef,
-                    loader_path    => mark_raw( to_json( $loader_path, readable => $self->{app}->{devel} )->$* ),
+                    loader_path    => to_json( $loader_path, readable => $self->{app}->{devel} )->$*,
                     app_namespace  => $Pcore::Ext::NS,
                     viewport_class => $ext_app->{viewport},
-                    static_classes => undef,                           # TODO
+                    overrides      => $overrides,
+                    static_classes => !$self->{app}->{devel} ? $ext_app->{js}->$* : undef,
                 },
             };
 
-            my $index = P->tmpl->render( 'ext/index.html', $data );
+            $self->{cache} = P->tmpl->render( 'ext/index.html', $data );
 
-            $req->( 200, [ 'Content-Type' => 'text/html; charset=UTF-8' ], $index )->finish;
+            $req->( 200, [ 'Content-Type' => 'text/html; charset=UTF-8' ], $self->{cache} )->finish;
 
             return;
         }
@@ -137,18 +162,6 @@ around run => sub ( $orig, $self, $req ) {
 };
 
 1;
-## -----SOURCE FILTER LOG BEGIN-----
-##
-## PerlCritic profile "pcore-script" policy violations:
-## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
-## | Sev. | Lines                | Policy                                                                                                         |
-## |======+======================+================================================================================================================|
-## |    3 | 21                   | ValuesAndExpressions::ProhibitInterpolationOfLiterals - Useless interpolation of literal string                |
-## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    1 | 78                   | RegularExpressions::ProhibitEnumeratedClasses - Use named character classes ([a-z] vs. [[:lower:]])            |
-## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
-##
-## -----SOURCE FILTER LOG END-----
 __END__
 =pod
 
