@@ -1,4 +1,4 @@
-package Pcore::Ext v0.3.1;
+package Pcore::Ext v0.3.2;
 
 use Pcore -dist;
 use Pcore::Ext::Context;
@@ -189,60 +189,48 @@ sub SCAN ( $self, $app, $framework ) {
         $js_packer->minify( $ctx->{ctx}->{js}, { compress => 'obfuscate' } ) if !$app->{devel};    # clean
     }
 
-    # build apps content
-    # TODO remove deps from requires upon build
+    # build ext apps content
     if ( !$app->{devel} ) {
         for my $ext_app ( values $CFG->{app}->%* ) {
-            my ( %deps, %processed_class );
+            my ( %processed_class, @app_js );
 
-            # scan app deps
+            # topologically sort deps tree
             my $add_deps = sub ($class_name) {
-                return if exists $processed_class{$class_name};
+                if ( !exists $processed_class{$class_name} ) {
+                    $processed_class{$class_name} = 1;                                             # mark node as "gray"
+                }
+                elsif ( $processed_class{$class_name} == 1 ) {
+                    die q[Cyclic dependency found];                                                # entered to the "gray" node, this is cyclic deps
+                }
+                else {
+                    return;                                                                        # entered to the "black" node
+                }
 
                 my $class_cfg = $CFG->{class}->{$class_name};
 
-                return if !$class_cfg;
-
-                $processed_class{$class_name} = undef;
-
-                $deps{$class_name} = undef;
-
                 for my $require ( keys $class_cfg->{requires}->%* ) {
-                    next if !exists $CFG->{class}->{$require};
-
-                    $deps{$require} = undef;
+                    next if !exists $CFG->{class}->{$require};                                     # skip external deps
 
                     __SUB__->($require);
                 }
 
+                # all deps processed
+                $processed_class{$class_name} = 2;                                                 # mark node as "black"
+
+                # add content
+                push @app_js, $class_cfg->{js}->$*;
+
                 return;
             };
 
-            # build app deps map
+            # start sorting deps
             for my $class ( values $CFG->{class}->%* ) {
                 next if $class->{app_namespace} // $class->{namespace} ne $ext_app->{namespace};
 
                 $add_deps->( $class->{class} );
             }
 
-            my @app_content;
-
-            # build app deps
-            for my $class ( keys %deps ) {
-                my $ctx = $self->_get_ctx( $class, $app, $framework );
-
-                my @requires = grep { !exists $deps{$_} } keys $ctx->{ctx}->{requires}->%*;
-
-                my $app_content = $ctx->to_js( \@requires );
-
-                $js_packer->minify( $app_content, { compress => 'obfuscate' } );    # clean
-
-                push @app_content, $app_content->$*;
-            }
-
-            $ext_app->{js} = \join ';', @app_content;
-
-            $ext_app->{js}->$* .= ';';
+            $ext_app->{js} = \join ';', @app_js;
         }
     }
 
